

import java.util.Iterator;
 
import com.sonicsw.xq.*;


import java.sql.DriverManager;
import java.sql.Connection;

import java.sql.SQLException;

public class JSONProcessor implements XQService {


	
	// This is the XQLog (the container's logging mechanism).
    private XQLog m_xqLog = null;

    private Connection con = null;
    private String m_JDBC_URL =null;
    private String m_UserName=null;
    private String m_Password=null;

    /**
     * Constructor for a JSONProcessor
     */
	public JSONProcessor () {	
	}
	
	/**
     * Initialize the XQService by processing its initialization parameters.
     *
     * <p> This method implements a required XQService method.
     *
     * @param initialContext The Initial Service Context provides access to:<br>
     * <ul>
     * <li>The configuration parameters for this instance of the JSONProcessor.</li>
     * <li>The XQLog for this instance of the JSONProcessor.</li>
     * </ul>
     * @exception XQServiceException Used in the event of some error.
     */
    public void init(XQInitContext initialContext) throws XQServiceException {
   	 @SuppressWarnings("unused")	
    	XQParameters params = initialContext.getParameters();
        m_xqLog = initialContext.getLog();

        m_xqLog.logInformation("[JSONProcessor] Initializing ...");
        m_JDBC_URL = params.getParameter("JDBCURL",XQConstants.PARAM_STRING, null);
        m_UserName = params.getParameter("UserName",XQConstants.PARAM_STRING, null);
        m_Password = params.getParameter("Password",XQConstants.PARAM_STRING, null);
        
        m_xqLog.logInformation("[JSONProcessor].[Init]: JDBC URL=" + m_JDBC_URL.toString());
        m_xqLog.logInformation("[JSONProcessor].[Init]: User Name=" + m_UserName.toString());

        //Establish the JDBC Connection
        setConnection();

    }


    /**
     * Handle the arrival of XQMessages in the INBOX.
     *
     * <p> This method implement a required XQService method.
     *
     * @param ctx The service context.
     * @exception XQServiceException Thrown in the event of a processing error.
     */
    public void service(XQServiceContext ctx) throws XQServiceException {
        if (ctx == null)
            throw new XQServiceException("Service Context cannot be null.");
        else {

            // Get the message.
            XQEnvelope env = null;
            while (ctx.hasNextIncoming()) {
                env = ctx.getNextIncoming();
                if (env != null) {
                    XQMessage msg = env.getMessage();
                    try {
                        int iPartCnt = msg.getPartCount();
                        for (int i=0; i< iPartCnt; i++) {
                            XQPart prt = msg.getPart(i);
                            // Extract the parameters from the message part
                            //  Process the input message                           
                            //AreaDimXMLHandler handler = new AreaDimXMLHandler();
                            JSONHandler handler = new JSONHandler();

                            int recordCount =0;                         
                            int retryCount =3; //Retry for deadlocks and stale connections
                            boolean needNewConnection=false;
                            boolean transactionCompleted = false;
                          
                            do{
	                            //Try to handle XML
	                            try {
	                            	//If the connection is null, try to establish it.
	                            	if (needNewConnection){
	                            		setConnection();
	                            	}
	                            	recordCount = handler.handleJSON(prt,con);            
	                                if (recordCount==-1){
	                                	m_xqLog.logError("[JSONProcessor].[Service]: No records was processed"); 
	                                     throw new XQServiceException("[JSONProcessor].[Service]: No records was processed");
	                                }else{
	                                    m_xqLog.logInformation("[JSONProcessor].[Service]:Inserted -"+ recordCount+ "- records!");               
	                                }
                                    transactionCompleted = true;
	                            } catch (SQLException sqlEx) {
	                                //
	                                // The two SQL states that are 'retry-able' are 08S01
	                                // for a communications error, and 40001 for deadlock.
	                                //
	                                // Only retry if the error was due to a stale connection,
	                                // communications problem or deadlock
	                                //
	                                String sqlState = sqlEx.getSQLState();
	                                if (sqlState.equals(DBXMLHandler.SQL_STALE_CONNECTION) || sqlState.equals(DBXMLHandler.SQL_DEADLOCK)) {
	                                    retryCount--;
	                                    needNewConnection=true;
	                                    m_xqLog.logInformation("[JSONProcessor].[Service]:Encountered a bad connection or deadlock. SQL State ="+ sqlState);               

	                                } else {
	                            		m_xqLog.logError("[JSONProcessor].[Service]: SQLException encountered. SQL Code=" + sqlEx.getErrorCode() + " SQL State="+ sqlEx.getSQLState()); 
	                                	throw new XQServiceException(sqlEx);
	                                }
	                            }catch(Exception ex){
	                        		m_xqLog.logError("[JSONProcessor].[Service]: Exception thrown from XML Handler."); 
	                            	throw new XQServiceException(ex);
	                        	}
                            } while (!transactionCompleted && retryCount > 0);                    
                        
                        }
                    } catch (XQMessageException me) {
                        throw new XQServiceException("[JSONProcessor].[Service]: Exception accessing XQMessage: " + me.getMessage(), me);
                    }

                    // Pass message onto the outbox.
                    Iterator addressList = env.getAddresses();
                    if (addressList.hasNext())  {
                        // Add the message to the Outbox
                        ctx.addOutgoing(env);
                    }
                }
            }
        }
    }
    
    /**
     * Clean up and get ready to destroy the service.
     *
     * <p> This method implement a required XQService method.
     */
    public void destroy() {
       	if (con!=null){

	    	try {
	        	con.close();          
	        } catch (SQLException se) {
	          	m_xqLog.logError("[JSONProcessor].[Destroy]:Problem closing the connection");
	          	m_xqLog.logError("[JSONProcessor].[Destroy]: Stack Trace..." + se.toString());
	        }
       	}
    }// Destroy End
    
    /**
     * Set the JDBC connection
     *
     */
    private void setConnection() throws XQServiceException{
         m_xqLog.logInformation("[JSONProcessor].[setConnection]:Checking if Driver is registered with DriverManager.");
        
        try {
          Class.forName("com.mysql.jdbc.Driver");
        } catch (ClassNotFoundException cnfe) {
        	m_xqLog.logError("[JSONProcessor].[setConnection]: Couldn't find the driver!");
        	m_xqLog.logError("[JSONProcessor].[setConnection]:Let's print a stack trace, and exit.");
        	m_xqLog.logError("[JSONProcessor].[setConnection]:Stack Trace..." + cnfe.toString());

            throw new XQServiceException(cnfe);
        
        }
        
        m_xqLog.logInformation("[JSONProcessor].[setConnection]: Registered the driver ok!!!.......");
            
        try {
          // The second and third arguments are the username and password,
          // respectively. They should be whatever is necessary to connect
          // to the database.
     	   con = DriverManager.getConnection(m_JDBC_URL,m_UserName,m_Password);
        } catch (SQLException se) {
        	m_xqLog.logError("[JSONProcessor].[setConnection]: Couldn't connect: print out a stack trace and exit.");
        	m_xqLog.logError("[JSONProcessor].[setConnection]: Let's print a stack trace, and exit.");
        	m_xqLog.logError("[JSONProcessor].[setConnection]: Stack Trace..." + se.toString());

            throw new XQServiceException(se);
        }
        
        if (con != null)
        	m_xqLog.logInformation("[JSONProcessor].[setConnection]: Hooray! We connected to the database!");
        else
        	m_xqLog.logInformation("[JSONProcessor].[setConnection]: We should never get here.");
       
   }

}